data_dir <- file.path("..", "testdata")
tempfile_nc <- function() {
  tempfile_helper("runsd_")
}

########## simple case ##########
file_out <- tempfile_nc()
runsd("SIS", nts=2,
       file.path(data_dir, "ex_run.nc"),
       file_out)
file <- nc_open(file_out)

test_that("data is correct", {
  actual <- ncvar_get(file, "SIS")

  expected_data <- c(
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767,
    17.67767,17.67767,17.67767,17.67767,17.67767
  )
  expected <- array(expected_data, c(5, 5, 2))

  expect_equivalent(actual, expected, tolerance = 1e-4)
})

test_that("attributes are correct", {
  actual <- ncatt_get(file, "lon", "units")$value
  expect_equal(actual, "degrees_east")

  actual <- ncatt_get(file, "lon", "long_name")$value
  expect_equal(actual, "longitude")

  actual <- ncatt_get(file, "lon", "standard_name")$value
  expect_equal(actual, "longitude")

  actual <- ncatt_get(file, "lon", "axis")$value
  expect_equal(actual, "X")

  actual <- ncatt_get(file, "lat", "units")$value
  expect_equal(actual, "degrees_north")

  actual <- ncatt_get(file, "lat", "long_name")$value
  expect_equal(actual, "latitude")

  actual <- ncatt_get(file, "lat", "standard_name")$value
  expect_equal(actual, "latitude")

  actual <- ncatt_get(file, "lat", "axis")$value
  expect_equal(actual, "Y")

  actual <- ncatt_get(file, "time", "units")$value
  expect_equal(actual, "hours since 1983-01-01 00:00:00")

  actual <- ncatt_get(file, "time", "long_name")$value
  expect_equal(actual, "time")

  actual <- ncatt_get(file, "time", "standard_name")$value
  expect_equal(actual, "time")

  actual <- ncatt_get(file, "time", "calendar")$value
  expect_equal(actual, "standard")

  actual <- ncatt_get(file, "SIS", "standard_name")$value
  expect_equal(actual, "SIS_standard")

  actual <- ncatt_get(file, "SIS", "long_name")$value
  expect_equal(actual, "Surface Incoming Shortwave Radiation")

  actual <- ncatt_get(file, "SIS", "units")$value
  expect_equal(actual, "W m-2")

  actual <- ncatt_get(file, "SIS", "_FillValue")$value
  expect_equal(actual, -999)

  actual <- ncatt_get(file, "SIS", "cmsaf_info")$value
  expect_equal(actual, "cmsafops::runsd for variable SIS")

  global_attr <- ncatt_get(file, 0)
  expect_equal(length(global_attr), 1)

  actual <- names(global_attr[1])
  expect_equal(actual, "Info")

  actual <- global_attr[[1]]
  expect_equal(actual, "Created with the CM SAF R Toolbox.")
})
